<html>
    <head>
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css" integrity="sha384-Vkoo8x4CGsO3+Hhxv8T/Q5PaXtkKtu6ug5TOeNV6gBiFeWPGFN9MuhOf23Q9Ifjh" crossorigin="anonymous">
        <style>
            div.group {
                border: 1px solid #DDD;
            }
        </style>
    </head>
    <body>
        <div id="content">
            <h1>List of available groups on your SQUAD instance</h1>
            <div id="group-list">
                {% for group in groups %}
                <div class="group">
                    <span>{{ group.slug }}</span>
                    <p>{{ group.description }}</p>
                </div>
                {% endfor %}
            </div>
        </div>
    </body>
</html>
