# Maximum number of objects to fetch from a unique endpoint
# this will avoid infinite loop
MAX_NUM_OF_OBJECTS = 10e6

# Default number of objects to load for a endpoint
DEFAULT_NUM_OF_OBJECTS = 50

# Maximum number of objects loaded per page in SQUAD
SQUAD_MAX_PAGE_LIMIT = 1000
